/*
 * [PrepareUnicodeRanges.java]
 *
 * Summary: Prepare table about Unicode 6 ranges.
 *
 * Copyright: (c) 2011-2017 Roedy Green, Canadian Mind Products, http://mindprod.com
 *
 * Licence: This software may be copied and used freely for any purpose but military.
 *          http://mindprod.com/contact/nonmil.html
 *
 * Requires: JDK 1.8+
 *
 * Created with: JetBrains IntelliJ IDEA IDE http://www.jetbrains.com/idea/
 *
 * Version History:
 *  1.0 2011-02-27 initial version
 */
package com.mindprod.unicode;

import com.mindprod.common18.BigDate;
import com.mindprod.common18.Build;
import com.mindprod.common18.EIO;
import com.mindprod.common18.ST;
import com.mindprod.csv.CSVReader;
import com.mindprod.fastcat.FastCat;
import com.mindprod.http.Get;

import java.io.EOFException;
import java.io.File;
import java.io.IOException;
import java.io.PrintWriter;
import java.net.URL;

import static java.lang.System.*;

/**
 * Prepare table about Unicode 6 ranges.
 *
 * @author Roedy Green, Canadian Mind Products
 * @version 1.0 2011-02-27 initial version
 * @since 2011-02-27
 */
public class PrepareUnicodeRanges
    {
    /**
     * reads E:/com/mindprod/unicode/unicoderanges.csn
     * put output directly into
     * e:/mindprod/jgloss/include/unicoderanges.htmlfrag
     * encoding for UTF-8
     */
    /**
     * main method
     *
     * @param args not used
     *
     * @throws java.io.IOException if IO trouble
     */
    public static void main( final String[] args ) throws IOException
        {
        // O P E N
        final File inFile = new File( Build.MINDPROD_SOURCE + "/unicode/unicoderanges.csv" );
        final CSVReader r = new CSVReader( EIO.getBufferedReader( inFile, 32 * 1024, EIO.UTF8 ) );
        final File outFile = new File( Build.MINDPROD_WEBROOT + "/jgloss/include/unicoderanges.htmlfrag" );
        final PrintWriter w = EIO.getPrintWriter( outFile, 32 * 1024, EIO.UTF8 );
        w.println( "<!-- This table was generated on " + BigDate.localToday() + " by com.mindprod.unicode.PrepareUnicodeRanges. D o   N o t   E d i t . -->" );
        try
            {
            while ( true )
                {
                // we just produce table rows.  Includer must provide headers.
                final int startOfRange = r.getHexInt();
                int version = r.getInt();
                int size = r.getInt();
                final String sampleGlyph = r.get();
                final String use = r.get();
                final String desc = r.get();
                if ( desc != null )
                    {
                    r.skipToNextLine();
                    }
                final String displayHex = ( startOfRange > 0xffff ? ST.toLZHexString( startOfRange,
                        8 ) : ST.toLZHexString( startOfRange, 4 ) ).toUpperCase();
                final String pdf;
                if ( startOfRange <= 0xffff )
                    {
                    pdf = 'U' + ST.toLZHexString( startOfRange, 4 ).toUpperCase() + ".pdf";
                    }
                else if ( startOfRange <= 0xfffff )
                    {
                    pdf = 'U' + ST.toLZHexString( startOfRange, 5 ).toUpperCase() + ".pdf";
                    }
                else if ( startOfRange <= 0xffffff )
                    {
                    pdf = 'U' + ST.toLZHexString( startOfRange, 6 ).toUpperCase() + ".pdf";
                    }
                else if ( startOfRange <= 0xfffffff )
                    {
                    pdf = 'U' + ST.toLZHexString( startOfRange, 7 ).toUpperCase() + ".pdf";
                    }
                else
                    {
                    pdf = 'U' + ST.toLZHexString( startOfRange, 8 ).toUpperCase() + ".pdf";
                    }
                if ( size == 0 )
                    {
                    final Get p = new Get();
                    final String response = p.send( new URL( "http://www.unicode.org/charts/PDF/" + pdf ),
                            Get.UTF8 );
                    size = response.length();
                    out.println( displayHex + ", " + size );
                    }
                final FastCat sb = new FastCat( 23 );
                // link via hex range
                sb.append( "<tr><td class=\"hex\">" );
                if ( version == 6 )
                    {
                    sb.append( "&#x2076;" );
                    }
                sb.append( "<a class=\"pdf\" href=\"http://www.unicode.org/charts/PDF/" );
                sb.append( pdf );
                sb.append( "\">" );
                sb.append( displayHex );
                sb.append( "</a></td>" );
                // size of pdf in k
                sb.append( "<td style=\"text-align:right\" class=\"unobtrusive\">" );
                sb.append( ( size + 1023 ) / 1024 );
                sb.append( "k</td>" );
                // sample glyph
                if ( sampleGlyph.endsWith( ".png" ) )
                    {  // in form of a png image. sampleGlyph
                    sb.append( "<td style=\"text-align:center\"><!-- macro Image unicode/" );
                    sb.append( sampleGlyph );
                    sb.append( " {sample glyph from range} --></td>" );
                    }
                else if ( sampleGlyph.length() > 0 )
                    {
                    // in form of an entity.
                    sb.append( "<td class=\"behold\">" );
                    sb.append( sampleGlyph );
                    sb.append( "</td>" );
                    }
                else
                    {
                    // no glyph.
                    sb.append( "<td></td>" );
                    }
                // use:desc
                sb.append( "<td><a class=\"pdf\" href=\"http://www.unicode.org/charts/PDF/" );
                sb.append( pdf );
                sb.append( "\">" );
                sb.append( use );
                sb.append( "</a>" );
                if ( desc != null )
                    {
                    sb.append( ": " );
                    sb.append( desc );
                    }
                sb.append( "</td></tr>" );
                w.println( sb.toString() );
                }
            }
        catch ( EOFException e )
            {
            r.close();
            w.close();
            }
        }
    }
